import re

from decimal import Decimal

import emoji as emojilib

from .emoji_data import emoji_text
from ..objects.emoji import _EmojiInt, _EmojiQual


class EmojiData:
    _UNSUPPORTED_NAME = ['handshake', 'people with bunny ears', 'men with bunny ears', 'women with bunny ears',
                         'people wrestling', 'men wrestling', 'women wrestling',
                         'woman and man holding hands', 'men holding hands', 'women holding hands',
                         'people holding hands', 'kiss', 'couple with heart', 'family']

    _UNSUPPORTED_MODIFIER = ['beard', 'bald', 'blond hair', 'white hair', 'curly hair', 'red hair']

    def __init__(self):
        data = {}
        group = None
        subgroup = None
        for line in emoji_text.split('\n'):
            line = line.strip()
            if line == '':
                continue
            elif line.startswith('#'):
                if line.startswith('# group:'):
                    group = line[9:].strip()
                    data[group] = {}
                elif line.startswith('# subgroup:'):
                    subgroup = line[12:].strip()
                    data[group][subgroup] = []
            else:
                emoji = EmojiData._parse_emoji_row(row=line)
                if emoji is not None and emoji.qual is _EmojiQual.FullyQualified:
                    data[group][subgroup].append(emoji)
        self.data = data

    def _parse_emoji_row(row: str) -> _EmojiInt:
        characters = row[:55].strip().split(' ')
        emoji = ''.join([chr(int(c, 16)) for c in characters])
        qualification = _EmojiQual(row[56:76].strip())
        version = Decimal(re.search(r'E\d+\.\d+', row[76:].strip()).group(0)[1:])
        raw_name = re.search(r'E\d+\.\d+(.*)', row[76:].strip()).group(1).strip()
        split_name = raw_name.split(': ')

        if len(split_name) > 1:
            if split_name[0] in ['flag', 'keycap']:
                name = ": ".join(split_name)
                child_of = None
            else:
                split_name = [split_name[0]] + [s.strip() for s in split_name[1].split(',')]
                if split_name[0] in EmojiData._UNSUPPORTED_NAME:
                    return None
                else:
                    for modifier in split_name[1:]:
                        if modifier in EmojiData._UNSUPPORTED_MODIFIER:
                            return None

                    name = ": ".join(split_name)
                    child_of = split_name[0]
        else:
            name = split_name[0]
            child_of = None

        return _EmojiInt(emoji=emoji, qual=qualification, version=version, name=name, child_of=child_of,
                         demojized=emojilib.demojize(emoji))



import enum

from dataclasses import dataclass


class _EmojiQual(str, enum.Enum):
    FullyQualified = "fully-qualified"
    MinimalQualified = "minimally-qualified"
    Unqualified = "unqualified"
    Component = "component"


class _EmojiVersion(str, enum.Enum):
    E1_0 = "1.0"
    E2_0 = "2.0"
    E3_0 = "3.0"
    E4_0 = "4.0"
    E5_0 = "5.0"
    E11_0 = "11.0"
    E12_0 = "12.0"
    E12_1 = "12.1"
    E13_0 = "13.0"
    E13_1 = "13.1"
    E14_0 = "14.0"
    E15_0 = "15.0"
    E15_1 = "15.1"


@dataclass(kw_only=True)
class _EmojiInt:
    emoji: str
    qual: _EmojiQual
    version: _EmojiVersion
    name: str
    child_of: str | None
    demojized: str


@dataclass(kw_only=True)
class Emoji:
    emoji: str
    name: str
    alias: str
    variants: list[str] | None = None



from decimal import Decimal

from .data.emojis import EmojiData
from .objects.emoji import Emoji, _EmojiVersion


class EmojiGroup:

    def __init__(self, data_block: dict):
        self.__parsed_data = data_block

    # Return a list of subgroups within the group
    @property
    def subgroups(self) -> list[str]:
        return list(self.__parsed_data.keys())

    # Return the specified subgroup
    def __getitem__(self, subgroup: str) -> list[Emoji]:
        return self.__parsed_data[subgroup]


class EmojiIndex:

    def __init__(self):
        self.__version = _EmojiVersion.E15_1
        self.__data = EmojiData()
        self.__load()

    # Build a modified version of EmojiData for the specified version
    def __load(self):
        self.__parsed_data = {}
        for group, data in self.__data.data.items():
            temp_group_data = {}
            for subgroup, data2 in data.items():
                temp_data = {}
                for emoji in data2:
                    if emoji.version <= Decimal(self.__version.value):
                        if emoji.child_of is None:
                            temp_data[emoji.name] = (
                                Emoji(emoji=emoji.emoji, name=emoji.name, alias=emoji.demojized))
                        else:
                            if temp_data[emoji.child_of].variants is None:
                                temp_data[emoji.child_of].variants = [emoji.emoji]
                            else:
                                temp_data[emoji.child_of].variants.append(emoji.emoji)
                if len(temp_data) > 0:
                    temp_group_data[subgroup] = list(temp_data.values())
            if len(temp_group_data) > 0:
                self.__parsed_data[group] = temp_group_data

    @property
    def version(self) -> _EmojiVersion:
        return self.__version

    def set_version(self, version: str):
        if version in _EmojiVersion.__members__.values():
            self.__version = _EmojiVersion(version)
            self.__load()
        else:
            raise ValueError(f"Version {version} not supported")

    # Return a list of groups
    @property
    def groups(self) -> list[str]:
        return list(self.__parsed_data.keys())

    # Return the specified group
    def __getitem__(self, group: str) -> EmojiGroup:
        return EmojiGroup(data_block=self.__parsed_data[group])
